# -*- coding: utf-8 -*-
#
# helper functions for raw export and raw te export
import datetime

from django.conf import settings
from django.shortcuts import render
from reportserver.lib.db.export_dal import get_export_data

from reportserver.lib.general_export import include_filter_options, \
    get_datepicker_context

from reportserver.lib.export.formats.csvexport import export_csv
from reportserver.lib.export.formats.excelexport import export_xlsx
from reportserver.lib.export.formats.zipexport import export_zip


def get_exportname_from(table):
    """
    Get exportname from table name.
    """
    if table == "eval_participations":
        return "Teilnahmen"
    elif table == "eval_virtual_participations":
        return "Teilnehmende"
    else:
        return table


def get_date_from(datestr):
    return datetime.datetime.strptime(datestr, "%d.%m.%Y").date()


def get_filter_type_name(string):
    str2name = {"1": u"Start und Ende innerhalb des Zeitraums",
                "2": u"Start innerhalb des Zeitraums",
                "3": u"Ende innerhalb des Zeitraums",
                "4": u"Start oder Ende innerhalb des Zeitraums",
                "5": u"Laufend innerhalb des Zeitraums"}
    return str2name.get(string)


def convert_list(list_, item_pairs):
    convert_map = {unicode(id): name for id, name in item_pairs}
    filtered_list = [x for x in list_ if x]
    if not filtered_list:
        return u"Keine Filterung"
    return u", ".join([convert_map[x] for x in filtered_list])


def get_metadata(request, table):
    return [["Programm", settings.PROGRAM],
            ["Datenstand", get_date_from(request.session["export_date"])],
            ["Exportdatum", datetime.date.today()],
            ["Filtertype", get_filter_type_name(
                request.POST.get('date_filter_type'))],
            ["Von", get_date_from(request.POST.get("startdate"))],
            ["Bis", get_date_from(request.POST.get("enddate"))],
            ["Bundesland", convert_list(request.POST.getlist("nuts1"),
                                        request.session["nuts1"])],
            ["Vorhaben", convert_list(request.POST.getlist("beneficiary"),
                                      request.session['beneficiary'])],
            ["Region", convert_list(request.POST.getlist("regioncategory"),
                                    request.session["regioncategory"])],
            ["Geschlecht", convert_list(request.POST.getlist("gender"),
                                        [("0", u"weiblich"),
                                         ("1", u"männlich")])]
            ]


def generate_export_for(request, table):
    """
    generates export for (virtual) participation
    :param request:
    :param table:
    :return:
    """
    rows = get_export_data(request, table)
    metadata = get_metadata(request, table)
    exportname = get_exportname_from(table)
    return export(request, rows, exportname, metadata)


def get_rawexport(request):
    """
    generates export for participations
    :param request:
    :return:
    """
    return generate_export_for(request, "eval_participations")


def get_raw_te_export(request):
    """
    generates export for virtual participations
    :param request:
    :return:
    """
    return generate_export_for(request, "eval_virtual_participations")


def export(request, rows, exportname, metadata):
    """
    does exports depending on defined export_type
    :param request:
    :param rows:
    :return:
    """
    export_type = request.POST.get("export_type")
    if export_type == "csv":
        return export_csv(rows, exportname)
    elif export_type == "xlsx":
        return export_xlsx(rows, exportname, metadata)
    elif export_type.startswith("zip"):
        document_number = export_type[3:]
        if document_number == "1" and exportname == "Teilnahmen":
            document_name = settings.AVAILABLE_DOCUMENTS.get(
                "participations",
                "Musterauswertung_Teilnahmen-leer.xlsx")
        elif document_number == "1" and exportname == "Teilnehmende":
            document_name = settings.AVAILABLE_DOCUMENTS.get(
                "participants",
                "Musterauswertung_Teilnehmende-leer.xlsx")
        return export_zip(rows, document_name, exportname, metadata)


def get_overview(request, template_name, url_name):
    """
    generates generically overview for participation export
    or export of virtual participations
    :param request:
    :param template_name:
    :param url_name:
    :return:
    """
    fslist = request.session.get("nuts1")
    rlist = request.session.get("regioncategory")
    blist = request.session.get("beneficiary")
    context = include_filter_options(get_datepicker_context(),
                                     url_name, fslist, rlist, blist)
    return render(request, template_name, context)
